@@ -1,9 +1,13 @@
+from geopandas._config import options  # noqa
+
 from geopandas.geoseries import GeoSeries  # noqa
 from geopandas.geodataframe import GeoDataFrame  # noqa
-from geopandas.array import _points_from_xy as points_from_xy  # noqa
+from geopandas.array import points_from_xy  # noqa
 
-from geopandas.io.file import read_file  # noqa
-from geopandas.io.sql import read_postgis  # noqa
+from geopandas.io.file import _read_file as read_file  # noqa
+from geopandas.io.arrow import _read_parquet as read_parquet  # noqa
+from geopandas.io.arrow import _read_feather as read_feather  # noqa
+from geopandas.io.sql import _read_postgis as read_postgis  # noqa
 from geopandas.tools import sjoin  # noqa
 from geopandas.tools import overlay  # noqa
 from geopandas.tools._show_versions import show_versions  # noqa
@@ -12,7 +16,6 @@ from geopandas.tools import clip  # noqa
 
 import geopandas.datasets  # noqa
 
-from geopandas._config import options  # noqa
 
 # make the interactive namespace easier to use
 # for `from geopandas import *` demos.
