@@ -4,7 +4,6 @@ import numpy as np
 import pandas as pd
 from pandas import DataFrame, MultiIndex, Series
 
-from pyproj import CRS
 from shapely.geometry import box
 from shapely.geometry.base import BaseGeometry
 from shapely.ops import cascaded_union
@@ -12,17 +11,11 @@ from shapely.ops import cascaded_union
 import geopandas as gpd
 
 from .array import GeometryArray, GeometryDtype
+from .sindex import get_sindex_class, has_sindex
 
-try:
-    from rtree.core import RTreeError
-
-    HAS_SINDEX = True
-except ImportError:
-
-    class RTreeError(Exception):
-        pass
-
-    HAS_SINDEX = False
+# for backwards compat
+# this will be static (will NOT follow USE_PYGEOS changes)
+HAS_SINDEX = has_sindex()
 
 
 def is_geometry_type(data):
@@ -42,10 +35,12 @@ def _delegate_binary_method(op, this, other, *args, **kwargs):
     # type: (str, GeoSeries, GeoSeries) -> GeoSeries/Series
     this = this.geometry
     if isinstance(other, GeoPandasBase):
-        this, other = this.align(other.geometry)
+        if not this.index.equals(other.index):
+            warn("The indices of the two GeoSeries are different.")
+            this, other = this.align(other.geometry)
+        else:
+            other = other.geometry
 
-        if this.crs != other.crs:
-            warn("GeoSeries crs mismatch: {0} and {1}".format(this.crs, other.crs))
         a_this = GeometryArray(this.values)
         other = GeometryArray(other.values)
     elif isinstance(other, BaseGeometry):
@@ -100,23 +95,22 @@ class GeoPandasBase(object):
     _sindex_generated = False
 
     def _generate_sindex(self):
-        if not HAS_SINDEX:
-            warn("Cannot generate spatial index: Missing package `rtree`.")
-        else:
-            from geopandas.sindex import SpatialIndex
-
-            stream = (
-                (i, item.bounds, idx)
-                for i, (idx, item) in enumerate(self.geometry.iteritems())
-                if pd.notnull(item) and not item.is_empty
-            )
-            try:
-                self._sindex = SpatialIndex(stream)
-            # What we really want here is an empty generator error, or
-            # for the bulk loader to log that the generator was empty
-            # and move on. See https://github.com/Toblerity/rtree/issues/20.
-            except RTreeError:
-                pass
+        sindex_cls = get_sindex_class()
+        if sindex_cls is not None:
+            _sindex = sindex_cls(self.geometry)
+            if not _sindex.is_empty:
+                self._sindex = _sindex
+            else:
+                warn(
+                    "Generated spatial index is empty and returned `None`. "
+                    "Future versions of GeoPandas will return zero-length spatial "
+                    "index instead of `None`. Use `len(gdf.sindex) > 0` "
+                    "or `if gdf.sindex` instead of `if gd.sindex is not None` "
+                    "to check for empty spatial indexes.",
+                    FutureWarning,
+                    stacklevel=3,
+                )
+                self._sindex = None
         self._sindex_generated = True
 
     def _invalidate_sindex(self):
@@ -142,15 +136,16 @@ class GeoPandasBase(object):
 
         Returns None if the CRS is not set, and to set the value it
         :getter: Returns a ``pyproj.CRS`` or None. When setting, the value
-        can be anything accepted by :meth:`pyproj.CRS.from_user_input`,
+        can be anything accepted by
+        :meth:`pyproj.CRS.from_user_input() <pyproj.crs.CRS.from_user_input>`,
         such as an authority string (eg "EPSG:4326") or a WKT string.
         """
-        return self._crs
+        return self.geometry.values.crs
 
     @crs.setter
     def crs(self, value):
         """Sets the value of the crs"""
-        self._crs = None if not value else CRS.from_user_input(value)
+        self.geometry.values.crs = value
 
     @property
     def geom_type(self):
@@ -176,8 +171,33 @@ class GeoPandasBase(object):
 
     @property
     def is_empty(self):
-        """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
-        empty geometries."""
+        """
+        Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
+        empty geometries.
+
+        Examples
+        --------
+        An example of a GeoDataFrame with one empty point, one point and one missing
+        value:
+
+        >>> from shapely.geometry import Point
+        >>> d = {'geometry': [Point(), Point(2,1), None]}
+        >>> gdf = gpd.GeoDataFrame(d, crs="EPSG:4326")
+        >>> gdf
+                           geometry
+        0  GEOMETRYCOLLECTION EMPTY
+        1   POINT (2.00000 1.00000)
+        2                      None
+        >>> gdf.is_empty
+        0     True
+        1    False
+        2    False
+        dtype: bool
+
+        See Also
+        --------
+        GeoSeries.isna : detect missing values
+        """
         return _delegate_property("is_empty", self)
 
     @property
@@ -320,14 +340,14 @@ class GeoPandasBase(object):
             The GeoSeries (elementwise) or geometric object to test for
             equality.
         """
-        return _binary_op("equals", self, other)
+        return _binary_op("geom_equals", self, other)
 
     def geom_almost_equals(self, other, decimal=6):
         """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` if
         each geometry is approximately equal to `other`.
 
         Approximate equality is tested at all points to the specified `decimal`
-        place precision.  See also :meth:`equals`.
+        place precision.  See also :meth:`geom_equals`.
 
         Parameters
         ----------
@@ -336,12 +356,12 @@ class GeoPandasBase(object):
         decimal : int
             Decimal place presion used when testing for approximate equality.
         """
-        return _binary_op("almost_equals", self, other, decimal=decimal)
+        return _binary_op("geom_almost_equals", self, other, decimal=decimal)
 
     def geom_equals_exact(self, other, tolerance):
         """Return True for all geometries that equal *other* to a given
         tolerance, else False"""
-        return _binary_op("equals_exact", self, other, tolerance=tolerance)
+        return _binary_op("geom_equals_exact", self, other, tolerance=tolerance)
 
     def crosses(self, other):
         """Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
@@ -390,7 +410,14 @@ class GeoPandasBase(object):
         return _binary_op("intersects", self, other)
 
     def overlaps(self, other):
-        """Return True for all geometries that overlap *other*, else False"""
+        """Returns True for all geometries that overlap *other*, else False.
+
+        Parameters
+        ----------
+        other : GeoSeries or geometric object
+            The GeoSeries (elementwise) or geometric object to test if
+            overlaps.
+        """
         return _binary_op("overlaps", self, other)
 
     def touches(self, other):
@@ -430,6 +457,44 @@ class GeoPandasBase(object):
         """
         return _binary_op("within", self, other)
 
+    def covers(self, other):
+        """
+        Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
+        each geometry that is entirely covering `other`.
+
+        An object A is said to cover another object B if no points of B lie
+        in the exterior of A.
+
+        See
+        https://lin-ear-th-inking.blogspot.com/2007/06/subtleties-of-ogc-covers-spatial.html
+        for reference.
+
+        Parameters
+        ----------
+        other : Geoseries or geometric object
+            The Geoseries (elementwise) or geometric object to check is being covered.
+        """
+        return _binary_geo("covers", self, other)
+
+    def covered_by(self, other):
+        """
+        Returns a ``Series`` of ``dtype('bool')`` with value ``True`` for
+        each geometry that is entirely covered by `other`.
+
+        An object A is said to cover another object B if no points of B lie
+        in the exterior of A.
+
+        See
+        https://lin-ear-th-inking.blogspot.com/2007/06/subtleties-of-ogc-covers-spatial.html
+        for reference.
+
+        Parameters
+        ----------
+        other : Geoseries or geometric object
+            The Geoseries (elementwise) or geometric object to check is being covered.
+        """
+        return _binary_geo("covered_by", self, other)
+
     def distance(self, other):
         """Returns a ``Series`` containing the distance to `other`.
 
@@ -746,8 +811,8 @@ class GeoPandasBase(object):
         original index and a zero-based integer index that counts the
         number of single geometries within a multi-part geometry.
 
-        Example
-        -------
+        Examples
+        --------
         >>> gdf  # gdf is GeoSeries of MultiPoints
         0         MULTIPOINT (0 0, 1 1)
         1    MULTIPOINT (2 2, 3 3, 4 4)
@@ -788,6 +853,34 @@ class GeoPandasBase(object):
         """
         return _CoordinateIndexer(self)
 
+    def equals(self, other):
+        """
+        Test whether two objects contain the same elements.
+
+        This function allows two GeoSeries or GeoDataFrames to be compared
+        against each other to see if they have the same shape and elements.
+        Missing values in the same location are considered equal. The
+        row/column index do not need to have the same type (as long as the
+        values are still considered equal), but the dtypes of the respective
+        columns must be the same.
+
+        Parameters
+        ----------
+        other : GeoSeries or GeoDataFrame
+            The other GeoSeries or GeoDataFrame to be compared with the first.
+
+        Returns
+        -------
+        bool
+            True if all elements are the same in both objects, False
+            otherwise.
+        """
+        # we override this because pandas is using `self._constructor` in the
+        # isinstance check (https://github.com/geopandas/geopandas/issues/1420)
+        if not isinstance(other, type(self)):
+            return False
+        return self._data.equals(other._data)
+
 
 class _CoordinateIndexer(object):
     # see docstring GeoPandasBase.cx property above
